#!/bin/sh

# Functions to save parts of the uci config over defaultreset

# Before defaultreset:
# Collect raw output from 'uci show'
# Store it in several files

# After defaultreset and boot:
# Transform the outputs of 'uci show' into uci scripts.


SAVECFG=/rom/tmp/savecfg

SAVECFG_WIFI=$SAVECFG"_wifi"
SAVECFG_FW_REDIRECT=$SAVECFG"_fw_redirect"
SAVECFG_FW_PARENTAL=$SAVECFG"_fw_parental"
SAVECFG_PASSWD_USER=$SAVECFG"_passwd_user"
SAVECFG_SHADOW_USER=$SAVECFG"_shadow_user"
SAVECFG_ICE=$SAVECFG"_ice"
SAVECFG_ICE_SERVER=$SAVECFG"_ice_server"


savecfg_set_path()
{
	SAVECFG=${1}"/savecfg"

	SAVECFG_WIFI=$SAVECFG"_wifi"
	SAVECFG_FW_REDIRECT=$SAVECFG"_fw_redirect"
	SAVECFG_FW_PARENTAL=$SAVECFG"_fw_parental"
	SAVECFG_PASSWD_USER=$SAVECFG"_passwd_user"
	SAVECFG_SHADOW_USER=$SAVECFG"_shadow_user"
	SAVECFG_ICE=$SAVECFG"_ice"
	SAVECFG_ICE_SERVER=$SAVECFG"_ice_server"
}

savecfg_new_file()
{
	file=$1
	rm -f $file
	touch $file
}

savecfg_rm_files()
{
	local files="$@"
	if [ -z "$files" ] ; then
		files="$SAVECFG_WIFI $SAVECFG_FW_REDIRECT $SAVECFG_FW_PARENTAL"
	fi
	local file
	for file in $files ; do
		rm -f $file
	done
}

#
# functions to save the config sections before starting changing the overlay
# (before iup, defaultreset)
#

savecfg_save_wifi()
{
	echo "Saving wifi SSIDs and passwords..."

	savecfg_new_file $SAVECFG_WIFI

	uci show wireless | grep "wifi-iface" >> $SAVECFG_WIFI
}

savecfg_save_fw_redirect()
{
	echo "Saving firewall redirect rules..."

	savecfg_new_file $SAVECFG_FW_REDIRECT

	uci show firewall | grep redirect >> $SAVECFG_FW_REDIRECT
}

savecfg_save_fw_parental()
{
	echo "Saving firewall parental rules..."

	savecfg_new_file $SAVECFG_FW_PARENTAL

	local parentalrules=$(uci show firewall | grep -i "].parental=" | grep -o "rule\[.*\]")
	for rule in $parentalrules ; do
		uci show firewall | grep -F "$rule" >> $SAVECFG_FW_PARENTAL
	done
}

savecfg_save_passwd_user()
{
	echo "Saving user password..."

	savecfg_new_file $SAVECFG_PASSWD_USER
	savecfg_new_file $SAVECFG_SHADOW_USER

	grep "user:" /etc/passwd >$SAVECFG_PASSWD_USER
	grep "user:" /etc/shadow >$SAVECFG_SHADOW_USER
}

savecfg_save_ice()
{
	echo "Saving ice configuration..."

	cp /etc/config/ice $SAVECFG_ICE
	cp /etc/iopsys/server.ini $SAVECFG_ICE_SERVER
}


#
# functions to apply back the config sections after the changes to overlay are completed
# (after iup, defaultreset)
#

savecfg_apply_wifi()
{
	[ -s "$SAVECFG_WIFI" ] || return

	echo "Applying wifi SSIDs and passwords..." > /dev/console
	local index ifname network
	local newindex newifname newnetwork

	# loop through all the user wifi-ifaces
	for index in $(seq 0 99) ; do
		ifname=$(sed -n 's/.*wifi-iface\['$index'\]\.ifname='\''\(.*\)'\''/\1/p' $SAVECFG_WIFI )
		[ "$ifname" ] || break
		network=$(sed -n 's/.*wifi-iface\['$index'\]\.network='\''\(.*\)'\''/\1/p' $SAVECFG_WIFI )

		newindex=""
		newifname=""
		newnetwork=""

		# loop through all the newly configured wifi-ifaces
		for newindex in $(seq 0 99) ; do
			newifname=$( uci show wireless | grep wifi-iface | sed -n 's/.*wifi-iface\['$newindex'\]\.ifname='\''\(.*\)'\''/\1/p' )
			if [ -z "$newifname" ] ; then
				# no newifname means that all the wifi-ifaces have already been parsed
				newindex="-1"
				break
			fi
			newnetwork=$(uci show wireless | grep wifi-iface | sed -n 's/.*wifi-iface\['$newindex'\]\.network='\''\(.*\)'\''/\1/p' )

			# two wifi interfaces are the same if "ifname" and "network" are the same
			if [ "$newifname" == "$ifname" ] && [ "$newnetwork" == "$network" ] ; then
				break
			fi
		done

		# prepend "set" to each line NOT containing "]="
		# wireless.@wifi-iface[$index].<option>='<value>' becomes
		# set wireless.@wifi-iface[$newindex].<option>='<value>'
		sed -i '/.*]=.*/! s/\(.*\[\)'$index'\(\].*\)/set \1'$newindex'\2/' $SAVECFG_WIFI

		if [ "$newindex" == "-1" ] ; then
			# prepend "add" to each line containing "]="
			# AND keep only the package name (before the first dot) and section type (after equal)
			# wireless.@wifi-iface[0]=wifi-iface becomes
			# add wireless wifi-iface
			sed -i 's/\(.*\)\..*\['$index'\]=\(.*\)/add \1 \2/' $SAVECFG_WIFI
		else
			# no need to create a new uci section, just delete the line defining a new section
			sed -i '/\(.*\)\..*\['$index'\]=\(.*\)/d' $SAVECFG_WIFI
		fi

	done
	# commit at the end of the script
	[ -s $SAVECFG_WIFI ] && echo "commit" >> $SAVECFG_WIFI

	# run the uci script through uci batch to get the expected result
	cat $SAVECFG_WIFI | uci batch >/dev/null 2>&1

	savecfg_rm_files $SAVECFG_WIFI
}

savecfg_apply_fw_redirect()
{
	[ -s "$SAVECFG_FW_REDIRECT" ] || return

	echo "Applying firewall redirect rules..." > /dev/console

	# prepend "set" to each line NOT containing "]="
	# firewall.@redirect[0].enabled='1' becomes
	# set firewall.@redirect[0].enabled='1'
	sed -i '/.*]=.*/! s/.*/set \0/' $SAVECFG_FW_REDIRECT

	# prepend "add" to each line containing "]="
	# AND keep only the package name (before the first dot) and section type (after equal)
	# firewall.@redirect[0]=redirect becomes
	# add firewall redirect
	sed -i 's/\(.*\)\..*]=\(.*\)/add \1 \2/' $SAVECFG_FW_REDIRECT

	# change all the array indexes to [-1]
	sed -i 's/\[.\?.\?.\?.\?\]/[-1]/' $SAVECFG_FW_REDIRECT

	# commit at the end of the script
	[ -s $SAVECFG_FW_REDIRECT ] && echo "commit" >> $SAVECFG_FW_REDIRECT

	# run the uci script through uci batch to get the expected result
	cat $SAVECFG_FW_REDIRECT | uci batch >/dev/null 2>&1

	savecfg_rm_files $SAVECFG_FW_REDIRECT
}

savecfg_apply_fw_parental()
{
	[ -s "$SAVECFG_FW_PARENTAL" ] || return

	echo "Applying firewall parental rules..." > /dev/console

	# prepend "set" to each line NOT containing "]="
	# firewall.@redirect[0].enabled='1' becomes
	# set firewall.@redirect[0].enabled='1'
	sed -i '/.*]=.*/! s/.*/set \0/' $SAVECFG_FW_PARENTAL

	# prepend "add" to each line containing "]="
	# AND keep only the package name (before the first dot) and section type (after equal)
	# firewall.@redirect[0]=redirect becomes
	# add firewall redirect
	sed -i 's/\(.*\)\..*]=\(.*\)/add \1 \2/' $SAVECFG_FW_PARENTAL

	# change all the array indexes to [-1]
	sed -i 's/\[.\?.\?.\?.\?\]/[-1]/' $SAVECFG_FW_PARENTAL

	# commit at the end of the script
	[ -s $SAVECFG_FW_PARENTAL ] && echo "commit" >> $SAVECFG_FW_PARENTAL

	# run the uci script through uci batch to get the expected result
	cat $SAVECFG_FW_PARENTAL | uci batch >/dev/null 2>&1

	savecfg_rm_files $SAVECFG_FW_PARENTAL
}

savecfg_apply_passwd_user()
{
	[ -s "$SAVECFG_PASSWD_USER" ] || return

	echo "Applying user password..." > /dev/console

	local passwd_line=$(cat $SAVECFG_PASSWD_USER)
	local shadow_line=$(cat $SAVECFG_SHADOW_USER)

	if [ -n "$passwd_line" -a "$(echo $passwd_line | wc -l)" == "1" ] ; then
		sed -i '/user:/d' /etc/passwd
		echo $passwd_line >> /etc/passwd
	fi
	if [ -n "$shadow_line" -a "$(echo $shadow_line | wc -l)" == "1" ] ; then
		sed -i '/user:/d' /etc/shadow
		echo $shadow_line >> /etc/shadow
	fi

	savecfg_rm_files $SAVECFG_PASSWD_USER
	savecfg_rm_files $SAVECFG_SHADOW_USER

	# prevent the password to be overwritten by
	# passwords.user.password=$KEY
	# that comes from /rom/etc/config/passwords

	uci delete passwords.user.password
	uci commit passwords
}

savecfg_apply_ice()
{
	[ -s "$SAVECFG_ICE" ] || return

	echo "Applying ice configuration..." > /dev/console

	cp $SAVECFG_ICE /etc/config/ice
	cp $SAVECFG_ICE_SERVER /etc/iopsys/server.ini

	savecfg_rm_files $SAVECFG_ICE $SAVECFG_ICE_SERVER
}


#
# main functions
#

#save_config_before_overlay_rebuild
savecfg_save_config()
{
	local saves="$@"
	[ -z "$saves" ] && saves="wifi fw_redirect fw_parental passwd_user ice"

	for save in $saves ; do
		[ "$save" == "wifi" ] && savecfg_save_wifi
		[ "$save" == "fw_redirect" ] && savecfg_save_fw_redirect
		[ "$save" == "fw_parental" ] && savecfg_save_fw_parental
		[ "$save" == "passwd_user" ] && savecfg_save_passwd_user
		[ "$save" == "ice" ] && savecfg_save_ice
	done
}

#apply_config_after_overlay_rebuild
savecfg_apply_config()
{
	local applies="$@"

	for apply in $applies ; do
		[ "$apply" == "wifi" ] && savecfg_apply_wifi
		[ "$apply" == "fw_redirect" ] && savecfg_apply_fw_redirect
		[ "$apply" == "fw_parental" ] && savecfg_apply_fw_parental
		[ "$apply" == "passwd_user" ] && savecfg_apply_passwd_user
		[ "$apply" == "ice" ] && savecfg_apply_ice
	done
}

savecfg_get_filenames()
{
	echo "$(find $(dirname $SAVECFG) -name "$(basename $SAVECFG)_*")"
}
